% Fraunhofer diffraction of a circular aperture 
% Fig.1.5 in Digital Holography by T.-C. Poon & J.-P. Liu
close all; clear all;
lambda=0.6*10^-6;   % wavelength, unit:m
delta=80*lambda;    % sampling period, unit:m
z=1;              % propagation distance, unit:m
M=512;              % space size
 
c=1:M;
r=1:M;
[C, R]=meshgrid(c, r);
THOR=(((R-M/2-1).^2+(C-M/2-1).^2).^0.5)*delta;
OB=zeros(M);        % Object 
for a=1:M;
    for b=1:M;
         if THOR(a,b)<=5*10^-4;  % aperture radius unit:m
            OB(a,b)=1;
        end
    end
end
FD=fftshift(fft2(fftshift(OB)));
FD=abs(FD);
FD=FD/max(max(FD));
C=C*lambda*z/M/delta*1000;
R=R*lambda*z/M/delta*1000;

figure; mesh(R, C, FD);
figure; imshow(FD);
title('Modulus of the Fraunhofer diffraction pattern')
