% Simulation of the Fourier transform holography
% Example 3.3 in Digital Holography by T.-C. Poon & J.-P. Liu
 
% Input data, set parameters
clear all, close all;
Ii=imread('DH256.bmp');% object pattern, size 256*256 pixels
Ii=double(Ii);

M=512;
I=zeros(512);
I(128:383,128:383)=Ii;  % zero-padding
 
figure; imshow(mat2gray(abs(I)));
title('Object pattern')
axis off
 
 
 
% Produce the FT hologram
r=1:M;
c=1:M;
[C, R]=meshgrid(c, r);
 
O=fftshift(ifft2(fftshift(I)));
R=ones(512,512);
R=R*max(max(abs(O)));
R=R.*exp(2i*pi.*C/4); % tilted reference light
H=(abs(O+R)).^2; % FT hologram
 
figure; imshow(mat2gray(abs(H)));
title('Fourier transform hologram')
axis off
 
 
% Reconstruction
 
U=fftshift(ifft2(fftshift(H)));
 
figure; imshow(900.*mat2gray(abs(U)));
title('Reconstructed image')
axis off
