%Simulation of the rainbow holography
% Example 3.4 in Digital Holography by T.-C. Poon & J.-P. Liu
 
% Input data, set parameters
clear all, close all;
Ii=imread('DH256.bmp');% size 256*256 pixels, 8bit image
Ii=double(Ii);
PH=rand([256,256]);
Ii=Ii.*exp(2i*pi*PH); % add a random phase on the object pattern
 
% 1. Forward propagation to hologram 1
M=256;
z=30; %(cm, distance between object and observer
w=6500*10^-8;  %(cm,  wavelength)
delta=0.005; %cm, pixel pitch 50um
r=1:M;
c=1:M;
[C, R]=meshgrid(c, r);
p=exp(-2i*pi*z.*((1/w)^2-(1/M/delta)^2.*(C-M/2-1).^2-...
    (1/M/delta)^2.*(R-M/2-1).^2).^0.5);
A0=fftshift(ifft2(fftshift(Ii)));
Az=A0.*p;
E=fftshift(fft2(fftshift(Az)));
ES=zeros(512);
ES(246:265,128:383)=E(118:137,:); % A slit aperture 
 
% 2.Forward propagation to hologram 2
M=512;
r=1:M;
c=1:M;
[C, R]=meshgrid(c, r);
A1=fftshift(ifft2(fftshift(conj(ES))));
z=60; %  distance between hologram1 and hologram 2
p=exp(-2i*pi*z.*((1/w)^2-(1/M/delta)^2.*(C-M/2-1).^2-...
    (1/M/delta)^2.*(R-M/2-1).^2).^0.5);
Az1=A1.*p;
H=fftshift(fft2(fftshift(Az1))); % object light at the hologram plane
 
% 3.Reconstruction (650nm)
H2=zeros(1024);
H2(256:767,256:767)=conj(H);
M=1024;
r=1:M;
c=1:M;
[C, R]=meshgrid(c, r);
z=30;
p=exp(-2i*pi*z.*((1/w)^2-(1/M/delta)^2.*(C-M/2-1).^2-...
    (1/M/delta)^2.*(R-M/2-1).^2).^0.5);
A2=fftshift(ifft2(fftshift(H2)));
Az2=A2.*p;
R650=fftshift(fft2(fftshift(Az2)));
R650=(abs(R650)).^2;
R650=R650/max(max(R650));
R650=R650(384:639,384:639);
figure; imshow(R650);
title('Reconstructed image(650nm)')
axis off
 
Az2=Az2.*p; % at observer plane
S650=fftshift(fft2(fftshift(Az2)));
S650=(abs(S650)).^2;
S650=S650/max(max(S650));
S650=S650(384:639,384:639);
 
% Reconstruction (550nm)
w2=5500*10^-8;
H3=H2.*exp(2i*pi*sind(0.4)*(w-w2)/w/w2.*R*delta);
p=exp(-2i*pi*z.*((1/w2)^2-(1/M/delta)^2.*(C-M/2-1).^2-...
    (1/M/delta)^2.*(R-M/2-1).^2).^0.5);
Az3=fftshift(ifft2(fftshift(H3))).*p;
R550=fftshift(fft2(fftshift(Az3)));
R550=(abs(R550)).^2;
R550=R550/max(max(R550));
R550=R550(384:639,384:639);
figure; imshow(R550);
title('Reconstructed image (550nm)') 
 
Az3=Az3.*p;
S550=fftshift(fft2(fftshift(Az3)));
S550=(abs(S550)).^2;
S550=S550/max(max(S550));
S550=S550(384:639,384:639);
 
% Reconstruction (450nm)
w3=4500*10^-8;
H4=H2.*exp(2i*pi*sind(0.4)*(w-w3)/w/w3.*R*delta);
p=exp(-2i*pi*z.*((1/w3)^2-(1/M/delta)^2.*(C-M/2-1).^2-...
    (1/M/delta)^2.*(R-M/2-1).^2).^0.5);
Az4=fftshift(ifft2(fftshift(H4))).*p;
R450=fftshift(fft2(fftshift(Az4)));
R450=(abs(R450)).^2;
R450=R450/max(max(R450));
R450=R450(384:639,384:639);
figure; imshow(R450);
title('reconstructed image (450nm)')
 
Az4=Az4.*p;
S450=fftshift(fft2(fftshift(Az4)));
S450=(abs(S450)).^2;
S450=S450/max(max(S450));
S450=S450(384:639,384:639);
 
% color slit image
SLIT=zeros(256,256,3);
SLIT(:,:,1)=S650;
SLIT(:,:,2)=S550;
SLIT(:,:,3)=S450;
SLIT=uint8(SLIT.*500);
figure; image(SLIT)
title('Slit images')
axis off
axis equal
