% Rectangular function and its corresponding Fourier transform 
% Example 4.3 in Digital Holography by T.-C. Poon & J.-P. Liu
% The code is just to plot the (a) and (b) parts of Fig.4.5
clear all; close all;
 
N=64;                  %the total number of samples
L=10;                  %the half width of the rectangular function
dx=0.05;               %spatial resolution = 0.05 cm
x=-(N/2):(N/2)-1;    %N samples centered at N/2+1
x=x*dx;              
REC=zeros(1,N);
REC(1,(N/2+1-L):(N/2+L))=1; 
% locate the aperture with width 2L to the center of the domain
 
 
SP=fft(fftshift(REC));   
% first locate the center of the aperture to the fist point
% then calculate the spectrum of REC
SP=fftshift(SP);    % locate the spectrum to the center of the domain
SP=SP/(max(SP)); % normalization
SP=real(SP);
df=1/N/dx;  %frequency resolution
f=-(N/2):(N/2)-1;
f=f*df;    % the coordinate of spatial frequency centered at N/2+1
 
% produce the figures
figure(1)
subplot(1,2,1)
plot(x,REC)
axis([-inf, inf, -0.25, 1.1])
axis square
title('Rectangular function')
xlabel('cm')
 
 
subplot(1,2,2)
plot(f,SP)
axis([-inf, inf, -0.25, 1.1])
axis square
title('Spectrum')
xlabel('1/cm')
