% Simulation of the diffraction of a rectangular aperture using the ASM 
% Example 4.4 in Digital Holography by T.-C. Poon & J.-P. Liu 
clear all;close all;
lambda=0.633*10^-6; %wavelength, unit:m
delta=10*lambda; % sampling period, unit:m
z=0.2; % propagation distance; unit:m
M=512;  % space size
 
c=1:M;
r=1:M;
[C, R]=meshgrid(c, r);
OB=zeros(M);   
OB(246:267,246:267)=1;% create the rectangular aperture
SP_OB=fftshift(ifft2(fftshift(OB)));
deltaf=1/M/delta;% sampling period in the spectrum domain
c=1:M;
r=1:M;
[C, R]=meshgrid(c, r);
SFTF=exp(-2i*pi*z.*((1/lambda)^2-((R-M/2-1).*deltaf).^2-...
    ((C-M/2-1).*deltaf).^2).^0.5);
% the spatial frequency transfer function of propagation of light
 
DF=fftshift(fft2(fftshift(SP_OB.*SFTF)));
Intensity=mat2gray((abs(DF)).^2);
figure; imshow(Intensity);
title('Intensity of the diffraction pattern')
