% Demonstration of on-axis holography
% Example 4.7 in Digital Holography by T.-C. Poon & J.-P. Liu
clear all; close all;
 
%%Reading input bitmap file
I=imread('DH256.bmp','bmp');   %  original object, 8bit image
I=double(I);
 
% parameter setup
M=256;  
deltax=0.001; % pixel pitch 0.001 cm (10 um)
w=633*10^-8;  % wavelength 633 nm
z=25; % 25 cm, propagation distance
 
%Step 1: simulation of propagation using the ASM
r=1:M;
c=1:M;
[C, R]=meshgrid(c, r);
A0=fftshift(ifft2(fftshift(I)));
deltaf=1/M/deltax;
p=exp(-2i*pi*z.*((1/w)^2-((R-M/2-1).*deltaf).^2-...
    ((C-M/2-1).*deltaf).^2).^0.5);
Az=A0.*p;
EO=fftshift(fft2(fftshift(Az)));
 
%Step 2: interference at the hologram plane
AV=4*(min(min(abs(EO)))+max(max(abs(EO))));
% the amplitude of reference light 
% the visibility can be controlled by modifying the amplitude
IH=(EO+AV).*conj(EO+AV);  
 
 
figure; imshow(I);
title('Original object')
axis off
 
figure; imshow(mat2gray(IH));
title('Hologram')
axis off
 
SP=abs(fftshift(ifft2(fftshift(IH))));
figure; imshow(500.*mat2gray(SP));
title('Hologram spectrum')
axis off
 
%Step 3: reconstruction
A1=fftshift(ifft2(fftshift(IH)));
Az1=A1.*conj(p);
EI=fftshift(fft2(fftshift(Az1)));
EI=mat2gray(EI.*conj(EI));
figure; imshow(EI);
title('Reconstructed image')
axis off
