% Demonstration of 4-step PSH
% Example 5.1 in Digital Holography by T.-C. Poon & J.-P. Liu
% the code is based on the simulation of on-axis hologram (Ex.4.7)
clear all; close all;
 
%%Reading input bitmap file
I=imread('DH256.bmp','bmp');   %  original object, 8bit image
I=double(I);
 
% parameter setup
M=256;  
deltax=0.001; % pixel pitch 0.001 cm (10 um)
w=633*10^-8;  % wavelength 633 nm
z=25; % 25 cm, propagation distance
delta=pi/2; % phase step

%Step 1: simulation of propagation using the ASM
r=1:M;
c=1:M;
[C, R]=meshgrid(c, r);
A0=fftshift(ifft2(fftshift(I)));
deltaf=1/M/deltax;
p=exp(-2i*pi*z.*((1/w)^2-((R-M/2-1).*deltaf).^2-...
    ((C-M/2-1).*deltaf).^2).^0.5);
Az=A0.*p;
EO=fftshift(fft2(fftshift(Az)));
 
%Step 2: Interference at the hologram plane
AV=(min(min(abs(EO)))+max(max(abs(EO))));
% the amplitude of reference light
 
% Recording of Four phase-shifting holograms
I0=(EO+AV).*conj(EO+AV);  
I1=(EO+AV*exp(-1j*delta)).*conj(EO+AV*exp(-1j*delta));
I2=(EO+AV*exp(-2j*delta)).*conj(EO+AV*exp(-2j*delta));
I3=(EO+AV*exp(-3j*delta)).*conj(EO+AV*exp(-3j*delta));
MAX=max(max([I0, I1, I2, I3]));
 
figure(1); imshow(I);
title('Original object')
axis off
 
figure(2)
subplot(2,2,1)
imshow(I0/MAX);
axis off
title('hologram 1')
 
subplot(2,2,2)
imshow(I1/MAX);
axis off
title('hologram 2')
 
subplot(2,2,3)
imshow(I2/MAX);
axis off
title('hologram 3')
 
subplot(2,2,4)
imshow(I3/MAX);
axis off
title('hologram 4')
 
 
%Step 3: Reconstruction
CH=(I0-I2)-1j*(I1-I3); % the complex hologram (4-step PSH)
A1=fftshift(ifft2(fftshift(CH)));
Az1=A1.*conj(p);
EI=fftshift(fft2(fftshift(Az1)));
EI=(EI.*conj(EI));
EI=EI/max(max(EI));
 
figure(3); 
imshow(EI);
title('Reconstructed image of 4-step PSH')
axis off
