% Demonstration of the out-of-focus haze
% Example 6.1 in Digital Holography by T.-C. Poon & J.-P. Liu

clear all; close all;
 
%%Reading input bitmap file
I1=zeros(512);
I1(220:300,220:230)=1;  % first object
I2=zeros(512);
I2(340:350,250:330)=1;  % second object
figure;imshow(I1);
title('Object 1')
axis off
 
figure;imshow(I2);
title('Object 2')
axis off
 
% parameter setup
M=512;  
deltax=0.001; % pixel pitch 0.001 cm (10 um)
w=633*10^-8;  % wavelength 633 nm
z1=30; % 25 cm, propagation distance z1
z2=15; % 15 cm, propagation distance z2
 
%Simulation of propagation using the ASM
r=1:M;
c=1:M;
[C, R]=meshgrid(c, r);
deltaf=1/M/deltax;
A01=fftshift(ifft2(fftshift(I1)));
p1=exp(-2i*pi*z1.*((1/w)^2-((R-M/2-1).*deltaf).^2-...
    ((C-M/2-1).*deltaf).^2).^0.5);
Az1=A01.*p1;
 
A02=fftshift(ifft2(fftshift(I2)));
p2=exp(-2i*pi*z2.*((1/w)^2-((R-M/2-1).*deltaf).^2-...
    ((C-M/2-1).*deltaf).^2).^0.5);
Az2=A02.*p2;
 
Hologram=fftshift(fft2(fftshift(Az1)))+fftshift(fft2(fftshift(Az2)));
Hr=mat2gray(real(Hologram)); % Real part of the complex hologram
Hi=mat2gray(imag(Hologram)); % Imaginary part of the complex hologram
 
figure;imshow (Hr)
title('Real part of the complex hologram')
axis off
figure;imshow (Hi)
title('Imaginary part of the complex hologram')
axis off
 
%Reconstruction
Ar=fftshift(ifft2(fftshift(Hologram)));
Arz1=Ar.*conj(p1);
EI1=fftshift(fft2(fftshift(Arz1)));
EI1=mat2gray(EI1.*conj(EI1));
figure; imshow(EI1);
title('Reconstructed image at z=z_1')
axis off
 
Arz2=Ar.*conj(p2);
EI2=fftshift(fft2(fftshift(Arz2)));
EI2=mat2gray(EI2.*conj(EI2));
figure; imshow(EI2);
title('Reconstructed image at z=z_2')
axis off
