% Two-wavelength optical contouring 
% Example 6.3 in Digital Holography by T.-C. Poon & J.-P. Liu
clear all; close all;
x=1:1:1000;
h=ones(1,1000);
h=700-h.*((x-201).^2)/1000;
h=h-min(h);  % Sample profile in um
 
L1=0.650;   % first wavelength in um
L2=0.652;   % second wavelength in um
Ls=L1*L2/(L2-L1); % synthetic wave length in um
 
%Phase function for the two wavelengths
PHASE1=4*pi*h/L1-4*pi*700/L1;
PHASE2=4*pi*h/L2-4*pi*700/L2;
 
 
FUN1=exp(1i*PHASE1);
WRAP_PHASE1=angle(FUN1);% wrapped phase function 1
FUN2=exp(1i*PHASE2);
WRAP_PHASE2=angle(FUN2);% wrapped phase function 2
 
% unwrapping phase function 1
UNWRAP_PHASE1=unwrap(WRAP_PHASE1,[],2);
 
% difference between two wrapped phase functions
PHASE3=WRAP_PHASE1-WRAP_PHASE2;
 
UNWRAP_PHASE3=unwrap(PHASE3,[],2);% unwrapping synthetic phase function
UNWRAP_PHASE3=UNWRAP_PHASE3-min(UNWRAP_PHASE3);
H=UNWRAP_PHASE3*Ls/4/pi; % calculate the corresponding height
 
figure; plot (h);
title('Sample profile')
xlabel('pixel');ylabel('height ( \mum )')
 
figure; plot (WRAP_PHASE1);
title('Wrapped phase 1')
xlabel('pixel');ylabel('Phase ( radian )')
 
figure; plot (UNWRAP_PHASE1);
title('Unwrapped phase 1')
xlabel('pixel');ylabel('Phase ( radian )')
 
figure; plot (PHASE3);
title(' Wrapped phase for synthetic wavelength')
xlabel('pixel');ylabel('Phase ( radian )')
 
figure; plot (UNWRAP_PHASE3);
title('Unwrapped phase for synthetic wavelength')
xlabel('pixel');ylabel('Phase ( radian )')
 
figure; plot (H);
title('Recovered profile of the sample')
xlabel('pixel');ylabel('height ( \mum )')
