%Generate two binary holograms to synthesize a display hologram 
%Fig. 7.20 in Digital Holography by T.-C. Poon & J.-P. Liu
clear all, close all;
 
I1=imread('FCU500.bmp'); % please load a 8bit, 500x500 pixels image
I1=double(I1);
 
delta=0.01266;   %sampling distance 12.66 um
lambda=0.000633; %wavelength 0.633 um
 
% add a random phase to the object
Rr=rand(500);
Rr=exp(1i*2*pi.*Rr);
I0=I1.*Rr;
 
figure; imshow(mat2gray(abs(I0)));
title('original object')
axis off
 
[M N]=size(I0);
z=-80+lambda/4; %(mm, distance)
r=1:M;
c=1:N;
[C, R]=meshgrid(c, r);
p=exp(-1i*2*pi*z.*((1/lambda)^2-(1/M/delta)^2.*(C-N/2-1).^2-...
    (1/N/delta)^2.*(R-M/2-1).^2).^0.5);
A0=fftshift(ifft2(fftshift(I0)));
Az=A0.*p;
E=fftshift(fft2(fftshift(Az))); % propagation
 
%binary hologram
Hr=real(E);
Hr=+(Hr>0);
Hi=imag(E);
Hi=+(Hi>0);
 
%reconstruction of the binary hologram Hr
Ir=fftshift(fft2(fftshift(conj(p)).*ifft2(fftshift(Hr))));
Ir=Ir.*conj(Ir);
Ir=Ir/(max(max(Ir)));
  
%reconstruction of the synthetic hologram
Hc=Hr+1j*Hi;
I=fftshift(fft2(fftshift(conj(p)).*ifft2(fftshift(Hc))));
I=I.*conj(I);
I=I./max(max(I));
  
figure;
imshow(mat2gray(Hr));
title('Binary hologram B_r')
axis off
 
figure;
imshow(mat2gray(Hi));
title('Binary hologram B_i')
axis off
 
figure;
imshow(3*Ir);
title('reconstructed image of B_r')
axis off
 
figure;
imshow(3*I);
title('Reconstructed image of the synthetic hologram')
axis off
